/*
Continuous Testing Toolkit
Copyright (C) 2009 Paul R. Holser, Jr.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.infinitest.toolkit.runners;

import static java.lang.reflect.Modifier.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.internal.builders.AllDefaultPossibilitiesBuilder;
import org.junit.runner.Runner;
import org.junit.runners.model.RunnerBuilder;

class EnclosedInnersRunnerBuilder extends RunnerBuilder {
    private final Class<?> enclosing;

    EnclosedInnersRunnerBuilder(Class<?> enclosing) {
        this.enclosing = enclosing;
    }

    @Override
    public Runner runnerForClass(Class<?> testClass) throws Throwable {
        if (enclosing.equals(testClass))
            return new AllDefaultPossibilitiesExceptAnnotationRunnerBuilder(true).runnerForClass(testClass);
        if (isStatic(testClass.getModifiers()))
            return new AllDefaultPossibilitiesBuilder(true).runnerForClass(testClass);
        return new BlockJUnit4InnerClassRunner(enclosing, testClass);
    }

    @Override
    public List<Runner> runners(Class<?> enclosingClass, Class<?>[] enclosedClass) {
        List<Runner> runners = new ArrayList<Runner>();

        runners.add(safeRunnerForClass(enclosing));
        for (Class<?> each : enclosedClass)
            runners.add(safeRunnerForClass(each));

        return runners;
    }
}
